@@ -27,76 +27,36 @@
 #  include "config.h"
 #endif
 
-#ifdef USE_SAMPLERATE
-#  include <samplerate.h>
-#endif
-
 #include "audconfig.h"
 
 #include "effect.h"
 #include "general.h"
 #include "playback.h"
+#include "playlist-new.h"
+#include "playlist-utils.h"
 #include "pluginenum.h"
+#include "plugin-registry.h"
 #include "util.h"
 #include "visualization.h"
 
 AudConfig cfg;
 
 AudConfig aud_default_config = {
-    .snap_distance = 10,
-    .use_realtime = FALSE,
     .shuffle = FALSE,
     .repeat = FALSE,
-    .scaled = FALSE,
-    .autoscroll = TRUE,
-    .analyzer_peaks = TRUE,
     .equalizer_autoload = FALSE,
-    .easy_move = FALSE,
     .equalizer_active = FALSE,
     .playlist_visible = FALSE,
     .equalizer_visible = FALSE,
     .player_visible = TRUE,
-    .player_shaded = FALSE,
-    .playlist_shaded = FALSE,
-    .equalizer_shaded = FALSE,
-    .allow_multiple_instances = FALSE,
-    .always_show_cb = TRUE,
-    .convert_underscore = TRUE,
-    .convert_twenty = TRUE,            /* convert %20 */
-    .convert_slash = TRUE,
     .show_numbers_in_pl = TRUE,
-    .snap_windows = TRUE,
-    .save_window_position = TRUE,
-    .dim_titlebar = TRUE,
-    .get_info_on_load = FALSE,         /* get playlist info on load */
-    .get_info_on_demand = TRUE,        /* get playlist info on demand */
-    .eq_scaled_linked = TRUE,
-    .sort_jump_to_file = FALSE,
-    .use_eplugins = FALSE,             /* use effect plugins */
-    .always_on_top = FALSE,
-    .sticky = FALSE,
     .no_playlist_advance = FALSE,
     .stopaftersong = FALSE,
-    .refresh_file_list = TRUE,
-    .use_pl_metadata = TRUE,
-    .warn_about_win_visibility = TRUE,
-    .use_backslash_as_dir_delimiter = FALSE,
-    .random_skin_on_play = FALSE,
-    .use_fontsets = FALSE,
-    .mainwin_use_bitmapfont = TRUE,
-    .allow_broken_skins = FALSE,
     .close_dialog_open = TRUE,
-    .close_dialog_add = TRUE,
     .equalizer_preamp = 0.0,
     .equalizer_bands = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
-    .scale_factor = 2.0,
-    .skin = NULL,
-    .outputplugin = NULL,
     .filesel_path = NULL,
     .playlist_path = NULL,
-    .playlist_font = NULL,
-    .mainwin_font = NULL,
-    .disabled_iplugins = NULL,         /* disabled input plugins */
     .enabled_gplugins = NULL,          /* enabled general plugins */
     .enabled_vplugins = NULL,          /* enabled visualization plugins */
     .enabled_eplugins = NULL,          /* enabled effect plugins */
@@ -104,57 +64,36 @@ AudConfig aud_default_config = {
     .eqpreset_default_file = NULL,
     .eqpreset_extension = NULL,
     .url_history = NULL,
-    .timer_mode = 0,
-    .playlist_position = 0,
-    .pause_between_songs_time = 2,
-    .pause_between_songs = FALSE,
-    .show_wm_decorations = FALSE,
-    .playlist_transparent = FALSE,
-    .eq_extra_filtering = TRUE,
     .resume_playback_on_startup = FALSE,
     .resume_state = 0,
     .resume_playback_on_startup_time = 0,
-    .show_separator_in_pl = TRUE,
     .chardet_detector = NULL,
     .chardet_fallback = NULL,
+    .chardet_fallback_s = NULL,
     .output_buffer_size = 500,
-    .playlist_detect = TRUE,           /* whether or not to postpone format detection on initial add */
     .show_filepopup_for_tuple = TRUE,
     .cover_name_include = NULL,        /* words identifying covers */
     .cover_name_exclude = NULL,        /* words that might not show up in cover names */
     .recurse_for_cover = FALSE,
     .recurse_for_cover_depth = 0,
-    .session_uri_base = NULL,          /* default session uri base (non-NULL = custom session uri base) */
     .filepopup_pixelsize = 150,        /* short side length of the picture in the filepopup */
     .filepopup_delay = 20,             /* delay until the filepopup comes up */
     .use_file_cover = FALSE,           /* use filename.jpg for coverart */
-    .use_extension_probing = TRUE,
-    .colorize_r = 255, .colorize_g = 255, .colorize_b = 255,
     .filepopup_showprogressbar = TRUE,
     .close_jtf_dialog = TRUE,          /* close jtf dialog on jump */
-    .twoway_scroll = TRUE,             /* use back and forth scroll */
     .software_volume_control = FALSE,
-    .warn_about_broken_gtk_engines = TRUE,
-    .disable_inline_gtk = FALSE,       /* disable inline themes */
     .remember_jtf_entry = TRUE,
     .output_bit_depth = 16,
     .enable_replay_gain = TRUE,
     .enable_clipping_prevention = TRUE,
     .replay_gain_track = TRUE,         /* track mode */
     .replay_gain_album = FALSE,        /* album mode */
-    .enable_adaptive_scaler = FALSE,
-    .replay_gain_preamp = 0.0,
-    .default_gain = -9.0,
-#ifdef USE_SAMPLERATE
-    .enable_src = FALSE,               /* enable resampling */
-    .src_rate = 48000,                 /* samplerate */
-    .src_type = SRC_SINC_BEST_QUALITY, /* default interpolation method */
-#endif
-    .bypass_dsp = FALSE,
-    .player_width = 450,
-    .player_height = 150,
+    .replay_gain_preamp = 0,
+    .default_gain = 0,
     .sw_volume_left = 100, .sw_volume_right = 100,
-    .no_dithering = 0,
+    .clear_playlist = FALSE,
+    .output_path = NULL,
+    .output_number = -1,
 };
 
 typedef struct aud_cfg_boolent_t {
@@ -175,148 +114,54 @@ typedef struct aud_cfg_strent_t {
     gboolean se_wrt;
 } aud_cfg_strent;
 
-
-/* XXX: case-sensitivity is bad for lazy nenolods. :( -nenolod */
-static gchar *pl_candidates[] = {
-    PLUGIN_FILENAME("ALSA"),
-    PLUGIN_FILENAME("coreaudio"),
-    PLUGIN_FILENAME("OSS"),
-    PLUGIN_FILENAME("sun"),
-    PLUGIN_FILENAME("ESD"),
-    PLUGIN_FILENAME("pulse_audio"),
-    PLUGIN_FILENAME("disk_writer"),
-    NULL
-};
-
-
 static aud_cfg_boolent aud_boolents[] = {
-    {"allow_multiple_instances", &cfg.allow_multiple_instances, TRUE},
-    {"use_realtime", &cfg.use_realtime, TRUE},
-    {"always_show_cb", &cfg.always_show_cb, TRUE},
-    {"convert_underscore", &cfg.convert_underscore, TRUE},
-    {"convert_twenty", &cfg.convert_twenty, TRUE},
-    {"convert_slash", &cfg.convert_slash, TRUE },
     {"show_numbers_in_pl", &cfg.show_numbers_in_pl, TRUE},
-    {"show_separator_in_pl", &cfg.show_separator_in_pl, TRUE},
-    {"snap_windows", &cfg.snap_windows, TRUE},
-    {"save_window_positions", &cfg.save_window_position, TRUE},
-    {"dim_titlebar", &cfg.dim_titlebar, TRUE},
-    {"get_info_on_load", &cfg.get_info_on_load, TRUE},
-    {"get_info_on_demand", &cfg.get_info_on_demand, TRUE},
-    {"eq_scaled_linked", &cfg.eq_scaled_linked, TRUE},
     {"no_playlist_advance", &cfg.no_playlist_advance, TRUE},
-    {"refresh_file_list", &cfg.refresh_file_list, TRUE},
-    {"sort_jump_to_file", &cfg.sort_jump_to_file, TRUE},
-    {"use_pl_metadata", &cfg.use_pl_metadata, TRUE},
-    {"warn_about_win_visibility", &cfg.warn_about_win_visibility, TRUE},
-    {"use_backslash_as_dir_delimiter", &cfg.use_backslash_as_dir_delimiter, TRUE},
-    {"player_shaded", &cfg.player_shaded, TRUE},
     {"player_visible", &cfg.player_visible, TRUE},
     {"shuffle", &cfg.shuffle, TRUE},
     {"repeat", &cfg.repeat, TRUE},
-    {"scaled", &cfg.scaled, TRUE},   /* toggles custom scale */
-    {"autoscroll_songname", &cfg.autoscroll, TRUE},
     {"stop_after_current_song", &cfg.stopaftersong, TRUE},
-    {"playlist_shaded", &cfg.playlist_shaded, TRUE},
     {"playlist_visible", &cfg.playlist_visible, TRUE},
-    {"use_fontsets", &cfg.use_fontsets, TRUE},
-    {"mainwin_use_bitmapfont", &cfg.mainwin_use_bitmapfont, TRUE},
     {"equalizer_visible", &cfg.equalizer_visible, TRUE},
     {"equalizer_active", &cfg.equalizer_active, TRUE},
-    {"equalizer_shaded", &cfg.equalizer_shaded, TRUE},
     {"equalizer_autoload", &cfg.equalizer_autoload, TRUE},
-    {"easy_move", &cfg.easy_move, TRUE},
-    {"use_eplugins", &cfg.use_eplugins, TRUE},
-    {"always_on_top", &cfg.always_on_top, TRUE},
-    {"sticky", &cfg.sticky, TRUE},
-    {"random_skin_on_play", &cfg.random_skin_on_play, TRUE},
-    {"pause_between_songs", &cfg.pause_between_songs, TRUE},
-    {"show_wm_decorations", &cfg.show_wm_decorations, TRUE},
-    {"eq_extra_filtering", &cfg.eq_extra_filtering, TRUE},
-    {"analyzer_peaks", &cfg.analyzer_peaks, TRUE},
-    {"allow_broken_skins", &cfg.allow_broken_skins, TRUE},
     {"close_dialog_open", &cfg.close_dialog_open, TRUE},
-    {"close_dialog_add", &cfg.close_dialog_add, TRUE},
     {"resume_playback_on_startup", &cfg.resume_playback_on_startup, TRUE},
-    {"playlist_detect", &cfg.playlist_detect, TRUE},
     {"show_filepopup_for_tuple", &cfg.show_filepopup_for_tuple, TRUE},
     {"recurse_for_cover", &cfg.recurse_for_cover, TRUE},
     {"use_file_cover", &cfg.use_file_cover, TRUE},
-    {"use_extension_probing", &cfg.use_extension_probing, TRUE},
     {"filepopup_showprogressbar", &cfg.filepopup_showprogressbar, TRUE},
     {"close_jtf_dialog", &cfg.close_jtf_dialog, TRUE},
-    {"twoway_scroll", &cfg.twoway_scroll, TRUE},
     {"software_volume_control", &cfg.software_volume_control, TRUE},
-    {"warn_about_broken_gtk_engines", &cfg.warn_about_broken_gtk_engines, TRUE},
-    {"disable_inline_gtk", &cfg.disable_inline_gtk, TRUE},
     {"remember_jtf_entry", &cfg.remember_jtf_entry, TRUE},
     {"enable_replay_gain", &cfg.enable_replay_gain, TRUE},
     {"enable_clipping_prevention", &cfg.enable_clipping_prevention, TRUE},
     {"replay_gain_track", &cfg.replay_gain_track, TRUE},
     {"replay_gain_album", &cfg.replay_gain_album, TRUE},
-    {"enable_adaptive_scaler", &cfg.enable_adaptive_scaler, TRUE},
-#ifdef USE_SAMPLERATE
-    {"enable_src", &cfg.enable_src, TRUE},
-#endif
-    {"bypass_dsp", &cfg.bypass_dsp, TRUE},
-    {"no_dithering", & cfg.no_dithering, 1},
+    {"clear_playlist", & cfg.clear_playlist, TRUE},
 };
 
 static gint ncfgbent = G_N_ELEMENTS(aud_boolents);
 
 static aud_cfg_nument aud_numents[] = {
-    {"player_x", &cfg.player_x, TRUE},
-    {"player_y", &cfg.player_y, TRUE},
-    {"timer_mode", &cfg.timer_mode, TRUE},
-    {"vis_type", &cfg.vis_type, TRUE},
-    {"analyzer_mode", &cfg.analyzer_mode, TRUE},
-    {"analyzer_type", &cfg.analyzer_type, TRUE},
-    {"scope_mode", &cfg.scope_mode, TRUE},
-    {"vu_mode", &cfg.vu_mode, TRUE},
-    {"voiceprint_mode", &cfg.voiceprint_mode, TRUE},
-    {"vis_refresh_rate", &cfg.vis_refresh, TRUE},
-    {"analyzer_falloff", &cfg.analyzer_falloff, TRUE},
-    {"peaks_falloff", &cfg.peaks_falloff, TRUE},
-    {"playlist_x", &cfg.playlist_x, TRUE},
-    {"playlist_y", &cfg.playlist_y, TRUE},
-    {"playlist_width", &cfg.playlist_width, TRUE},
-    {"playlist_height", &cfg.playlist_height, TRUE},
-    {"playlist_position", &cfg.playlist_position, TRUE},
-    {"equalizer_x", &cfg.equalizer_x, TRUE},
-    {"equalizer_y", &cfg.equalizer_y, TRUE},
-    {"snap_distance", &cfg.snap_distance, TRUE},
-    {"pause_between_songs_time", &cfg.pause_between_songs_time, TRUE},
     {"titlestring_preset", &cfg.titlestring_preset, TRUE},
-    {"resume_state", & cfg.resume_state, 1},
+    {"resume_state", & cfg.resume_state, TRUE},
     {"resume_playback_on_startup_time", &cfg.resume_playback_on_startup_time, TRUE},
     {"output_buffer_size", &cfg.output_buffer_size, TRUE},
     {"recurse_for_cover_depth", &cfg.recurse_for_cover_depth, TRUE},
     {"filepopup_pixelsize", &cfg.filepopup_pixelsize, TRUE},
     {"filepopup_delay", &cfg.filepopup_delay, TRUE},
-    {"colorize_r", &cfg.colorize_r, TRUE},
-    {"colorize_g", &cfg.colorize_g, TRUE},
-    {"colorize_b", &cfg.colorize_b, TRUE},
     {"output_bit_depth", &cfg.output_bit_depth, TRUE},
-#ifdef USE_SAMPLERATE
-    {"src_rate", &cfg.src_rate, TRUE},
-    {"src_type", &cfg.src_type, TRUE},
-#endif
-    {"player_width", &cfg.player_width, TRUE},
-    {"player_height", &cfg.player_height, TRUE},
-    {"sw_volume_left", & cfg.sw_volume_left, 1},
-    {"sw_volume_right", & cfg.sw_volume_right, 1},
+    {"sw_volume_left", & cfg.sw_volume_left, TRUE},
+    {"sw_volume_right", & cfg.sw_volume_right, TRUE},
+    {"output_number", & cfg.output_number, TRUE},
 };
 
 static gint ncfgient = G_N_ELEMENTS(aud_numents);
 
 static aud_cfg_strent aud_strents[] = {
-    {"playlist_font", &cfg.playlist_font, TRUE},
-    {"mainwin_font", &cfg.mainwin_font, TRUE},
     {"eqpreset_default_file", &cfg.eqpreset_default_file, TRUE},
     {"eqpreset_extension", &cfg.eqpreset_extension, TRUE},
-    {"skin", &cfg.skin, FALSE},
-    {"output_plugin", &cfg.outputplugin, FALSE},
-    {"disabled_iplugins", &cfg.disabled_iplugins, TRUE},
     {"enabled_gplugins", &cfg.enabled_gplugins, FALSE},
     {"enabled_vplugins", &cfg.enabled_vplugins, FALSE},
     {"enabled_eplugins", &cfg.enabled_eplugins, FALSE},
@@ -327,7 +172,7 @@ static aud_cfg_strent aud_strents[] = {
     {"chardet_fallback", &cfg.chardet_fallback, TRUE},
     {"cover_name_include", &cfg.cover_name_include, TRUE},
     {"cover_name_exclude", &cfg.cover_name_exclude, TRUE},
-    {"session_uri_base", &cfg.session_uri_base, TRUE}
+    {"output_path", & cfg.output_path, TRUE},
 };
 
 static gint ncfgsent = G_N_ELEMENTS(aud_strents);
@@ -337,30 +182,33 @@ static gboolean
 save_extra_playlist(const gchar * path, const gchar * basename,
         gpointer savedlist)
 {
-    GList *playlists, *iter;
+    gint playlists, playlist;
     GList **saved;
-    Playlist *playlist;
     int found;
-    gchar *filename;
+    const gchar * filename;
 
-    playlists = playlist_get_playlists();
+    playlists = playlist_count ();
     saved = (GList **) savedlist;
 
     found = 0;
-    for (iter = playlists; iter; iter = iter->next) {
-        playlist = (Playlist *) iter->data;
-        if (g_list_find(*saved, playlist)) continue;
-        filename = playlist_filename_get(playlist);
-        if (!filename) continue;
+
+    for (playlist = 0; playlist < playlists; playlist ++)
+    {
+        if (g_list_find (* saved, GINT_TO_POINTER (playlist)) != NULL)
+            continue;
+
+        filename = playlist_get_filename (playlist);
+
+        if (filename == NULL)
+            continue;
+
         if (strcmp(filename, path) == 0) {
             /* Save playlist */
             playlist_save(playlist, path);
-            *saved = g_list_prepend(*saved, playlist);
+            * saved = g_list_prepend (* saved, GINT_TO_POINTER (playlist));
             found = 1;
-            g_free(filename);
             break;
         }
-        g_free(filename);
     }
 
     if(!found) {
@@ -374,18 +222,22 @@ save_extra_playlist(const gchar * path, const gchar * basename,
 static void
 save_other_playlists(GList *saved)
 {
-    GList *playlists, *iter;
-    Playlist *playlist;
-    gchar *pos, *ext, *basename, *filename, *newbasename;
+    gint playlists, playlist;
+    gchar * pos, * ext, * basename, * newbasename, * new_filename;
+    const gchar * filename;
     int i, num, isdigits;
 
-    playlists = playlist_get_playlists();
-    for(iter = playlists; iter; iter = iter->next) {
-        playlist = (Playlist *) iter->data;
-        if (g_list_find(saved, playlist)) continue;
-        filename = playlist_filename_get(playlist);
-        if (!filename || !filename[0]
-                || g_file_test(filename, G_FILE_TEST_IS_DIR)) {
+    playlists = playlist_count ();
+
+    for (playlist = 0; playlist < playlists; playlist ++)
+    {
+        if (g_list_find (saved, GINT_TO_POINTER (playlist)) != NULL)
+            continue;
+
+        filename = playlist_get_filename (playlist);
+
+        if (filename == NULL || g_file_test (filename, G_FILE_TEST_IS_DIR))
+        {
             /* default basename */
 #ifdef HAVE_XSPF_PLAYLIST
             basename = g_strdup("playlist_01.xspf");
@@ -395,7 +247,7 @@ save_other_playlists(GList *saved)
         } else {
             basename = g_path_get_basename(filename);
         }
-        g_free(filename);
+
         if ((pos = strrchr(basename, '.'))) {
             *pos = '\0';
         }
@@ -420,9 +272,11 @@ save_other_playlists(GList *saved)
             }
         }
         /* attempt to generate unique filename */
-        filename = NULL;
+        new_filename = NULL;
+
         do {
-            g_free(filename);
+            g_free (new_filename);
+
             if (num < 0) {
                 /* try saving without number first */
                 newbasename = g_strdup_printf("%s%s", basename, ext);
@@ -436,14 +290,16 @@ save_other_playlists(GList *saved)
                     goto cleanup;
                 }
             }
-            filename = g_build_filename(aud_paths[BMP_PATH_PLAYLISTS_DIR],
+
+            new_filename = g_build_filename (aud_paths[BMP_PATH_PLAYLISTS_DIR],
                     newbasename, NULL);
             g_free(newbasename);
-        } while (g_file_test(filename, G_FILE_TEST_EXISTS));
+        }
+        while (g_file_test (new_filename, G_FILE_TEST_EXISTS));
 
-        playlist_save(playlist, filename);
+        playlist_save (playlist, new_filename);
 cleanup:
-        g_free(filename);
+        g_free (new_filename);
         g_free(basename);
     }
 }
@@ -462,6 +318,14 @@ aud_config_free(void)
   }
 }
 
+void aud_config_chardet_update(void)
+{
+    if (cfg.chardet_fallback_s != NULL)
+        g_strfreev(cfg.chardet_fallback_s);
+    cfg.chardet_fallback_s = g_strsplit_set(cfg.chardet_fallback, " ,:;|/", 0);
+}
+
+
 void
 aud_config_load(void)
 {
@@ -498,9 +362,6 @@ aud_config_load(void)
         cfg_db_get_float(db, NULL, eqtext, &cfg.equalizer_bands[i]);
     }
 
-    /* custom scale factor */
-    cfg_db_get_float(db, NULL, "scale_factor", &cfg.scale_factor);
-
     /* History */
     if (cfg_db_get_int(db, NULL, "url_history_length", &length)) {
         for (i = 1; i <= length; i++) {
@@ -518,50 +379,54 @@ aud_config_load(void)
 
     cfg_db_close(db);
 
-
-    if (cfg.playlist_font && strlen(cfg.playlist_font) == 0) {
-        g_free(cfg.playlist_font);
-        cfg.playlist_font = NULL;
-    }
-
-    if (cfg.mainwin_font && strlen(cfg.mainwin_font) == 0) {
-        g_free(cfg.mainwin_font);
-        cfg.mainwin_font = NULL;
-    }
-
     if (!cfg.gentitle_format)
         cfg.gentitle_format = g_strdup("${?artist:${artist} - }${?album:${album} - }${title}");
 
-    if (!cfg.outputplugin) {
-    gint iter;
-        gchar *pl_path = g_build_filename(PLUGIN_DIR, plugin_dir_list[0], NULL);
-
-        for (iter = 0; pl_candidates[iter] != NULL && cfg.outputplugin == NULL; iter++)
-    {
-         cfg.outputplugin = find_file_recursively(pl_path, pl_candidates[iter]);
-    }
-
-        g_free(pl_path);
-    }
-
     if (!cfg.chardet_detector)
         cfg.chardet_detector = g_strdup("");
 
     if (!cfg.chardet_fallback)
         cfg.chardet_fallback = g_strdup("");
 
+    aud_config_chardet_update();
+
     if (!cfg.cover_name_include)
         cfg.cover_name_include = g_strdup("");
 
     if (!cfg.cover_name_exclude)
         cfg.cover_name_exclude = g_strdup("back");
+}
 
-    if (!cfg.session_uri_base)
-        cfg.session_uri_base = g_strdup("");
+void save_all_playlists (void)
+{
+    GList * saved;
 
-    /* at least one of these should be true */
-    if ((!cfg.get_info_on_demand) && (!cfg.get_info_on_load))
-        cfg.get_info_on_demand = TRUE;
+    /* Main playlist becomes #0 at load, so save #0 as main. -jlindgren */
+    if (! playlist_save (0, aud_paths[BMP_PATH_PLAYLIST_FILE]))
+        g_warning ("Could not save main playlist\n");
+
+    /* Save extra playlists that were loaded from PLAYLISTS_DIR  */
+    saved = g_list_append (NULL, GINT_TO_POINTER (0));
+
+    if (! dir_foreach (aud_paths[BMP_PATH_PLAYLISTS_DIR], save_extra_playlist,
+     & saved, NULL))
+        g_warning ("Could not save extra playlists\n");
+
+    /* Save other playlists to PLAYLISTS_DIR */
+    save_other_playlists (saved);
+    g_list_free (saved);
+}
+
+static void save_output_path (void)
+{
+    const gchar * path = NULL;
+    gint type, number = -1;
+
+    if (current_output_plugin != NULL)
+        plugin_get_path (current_output_plugin, & path, & type, & number);
+
+    cfg.output_path = (path != NULL) ? g_strdup (path) : NULL;
+    cfg.output_number = number;
 }
 
 void
@@ -571,15 +436,13 @@ aud_config_save(void)
     gchar *str;
     gint i;
     mcs_handle_t *db;
-    GList *saved;
-    Playlist *playlist = playlist_get_active();
 
-    cfg.disabled_iplugins = input_stringify_disabled_list();
-
-    cfg.resume_state = playback_get_playing () ? playback_get_paused () ? 2 : 1
-     : 0;
+    cfg.resume_state = playback_get_playing () ? (playback_get_paused () ? 2 :
+     1) : 0;
     cfg.resume_playback_on_startup_time = playback_get_playing () ?
-     playback_get_time () / 1000 : 0;
+     playback_get_time () : 0;
+
+    save_output_path ();
 
     db = cfg_db_open();
 
@@ -595,13 +458,6 @@ aud_config_save(void)
                                aud_numents[i].ie_vname,
                                *aud_numents[i].ie_vloc);
 
-    /* This is a bit lame .. it'll end up being written twice,
-     * could do with being done a bit neater.  -larne   */
-    if (playlist) {
-        cfg_db_set_int(db, NULL, "playlist_position",
-                       playlist_get_position(playlist));
-    }
-
     hook_call("config save", db);
 
     for (i = 0; i < ncfgsent; ++i) {
@@ -623,12 +479,6 @@ aud_config_save(void)
         g_free(str);
     }
 
-    if (get_current_output_plugin())
-        cfg_db_set_string(db, NULL, "output_plugin",
-                              get_current_output_plugin()->filename);
-    else
-        cfg_db_unset_key(db, NULL, "output_plugin");
-
     str = general_stringify_enabled_list();
     if (str) {
         cfg_db_set_string(db, NULL, "enabled_gplugins", str);
@@ -670,19 +520,5 @@ aud_config_save(void)
 
     cfg_db_close(db);
 
-    if (!playlist_save(playlist, aud_paths[BMP_PATH_PLAYLIST_FILE])) {
-        g_warning("Could not save main playlist\n");
-    }
-
-    /* Save extra playlists that were loaded from PLAYLISTS_DIR  */
-    saved = NULL;
-    saved = g_list_prepend(saved, playlist); /* don't save default again */
-    if(!dir_foreach(aud_paths[BMP_PATH_PLAYLISTS_DIR], save_extra_playlist,
-            &saved, NULL)) {
-        g_warning("Could not save extra playlists\n");
-    }
-
-    /* Save other playlists to PLAYLISTS_DIR */
-    save_other_playlists(saved);
-    g_list_free(saved);
+    save_all_playlists ();
 }
