@@ -34,7 +34,7 @@ struct _GeneralPluginData {
 
 GList *get_general_list(void);
 GList *get_general_enabled_list(void);
-void enable_general_plugin(gint i, gboolean enable);
+void general_enable_plugin(GeneralPlugin *plugin, gboolean enable);
 gchar *general_stringify_enabled_list(void);
 void general_enable_from_stringified_list(const gchar * list);
 
