@@ -33,10 +33,6 @@
 
 #include <glib/gprintf.h>
 
-#ifdef USE_SAMPLERATE
-#  include <samplerate.h>
-#endif
-
 #ifdef USE_DBUS
 #  include "dbus-service.h"
 #  include "audctrl.h"
@@ -47,8 +43,6 @@
 #include "eggdesktopfile.h"
 #endif
 
-#include "libSAD.h"
-
 #include "build_stamp.h"
 #include "configdb.h"
 #include "equalizer.h"
@@ -57,25 +51,21 @@
 #include "logger.h"
 #include "output.h"
 #include "playback.h"
-#include "playlist.h"
+#include "playlist-new.h"
+#include "playlist-utils.h"
 #include "pluginenum.h"
-#include "scanner.h"
 #include "signals.h"
 #include "util.h"
-#include "vis_runner.h"
 #include "vfs.h"
 #include "chardet.h"
 
-#include "icons-stock.h"
-#include "images/audacious_player.xpm"
-
 #include "ui_headless.h"
-#include "ui_preferences.h"
 #include "ui_misc.h"
 
 static const gchar *application_name = N_("Audacious");
 
-struct _AudCmdLineOpt {
+struct _AudCmdLineOpt
+{
     gchar **filenames;
     gint session;
     gboolean play, stop, pause, fwd, rew, play_pause, show_jump_box;
@@ -89,24 +79,20 @@ struct _AudCmdLineOpt {
 };
 typedef struct _AudCmdLineOpt AudCmdLineOpt;
 
-static AudCmdLineOpt options = {};
+static AudCmdLineOpt options;
 
-gchar *aud_paths[BMP_PATH_COUNT] = {};
+gchar * aud_paths[BMP_PATH_COUNT];
 
 #ifdef USE_DBUS
 MprisPlayer *mpris;
 #endif
 
-static char start_playback = 0;
-
-static void
-print_version(void)
+static void print_version(void)
 {
-    g_printf("%s %s [%s]\n", _(application_name), VERSION, svn_stamp);
+    g_printf("%s %s [%s]\n", _(application_name), VERSION, build_stamp);
 }
 
-static void
-aud_make_user_dir(void)
+static void aud_make_user_dir(void)
 {
     const mode_t mode755 = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
 
@@ -117,10 +103,9 @@ aud_make_user_dir(void)
     make_directory(aud_paths[BMP_PATH_PLAYLISTS_DIR], mode755);
 }
 
-static void
-aud_free_paths(void)
+static void aud_free_paths(void)
 {
-    int i;
+    gint i;
 
     for (i = 0; i < BMP_PATH_COUNT; i++)
     {
@@ -129,54 +114,34 @@ aud_free_paths(void)
     }
 }
 
-static void
-aud_init_paths()
+static void aud_init_paths()
 {
-    char *xdg_config_home;
-    char *xdg_data_home;
-    char *xdg_cache_home;
+    gchar *xdg_config_home;
+    gchar *xdg_data_home;
+    gchar *xdg_cache_home;
 
-    xdg_config_home = (getenv("XDG_CONFIG_HOME") == NULL
-        ? g_build_filename(g_get_home_dir(), ".config", NULL)
-        : g_strdup(getenv("XDG_CONFIG_HOME")));
-    xdg_data_home = (getenv("XDG_DATA_HOME") == NULL
-        ? g_build_filename(g_get_home_dir(), ".local", "share", NULL)
-        : g_strdup(getenv("XDG_DATA_HOME")));
-    xdg_cache_home = (getenv("XDG_CACHE_HOME") == NULL
-        ? g_build_filename(g_get_home_dir(), ".cache", NULL)
-        : g_strdup(getenv("XDG_CACHE_HOME")));
+    xdg_config_home = (getenv("XDG_CONFIG_HOME") == NULL ? g_build_filename(g_get_home_dir(), ".config", NULL) : g_strdup(getenv("XDG_CONFIG_HOME")));
+    xdg_data_home = (getenv("XDG_DATA_HOME") == NULL ? g_build_filename(g_get_home_dir(), ".local", "share", NULL) : g_strdup(getenv("XDG_DATA_HOME")));
+    xdg_cache_home = (getenv("XDG_CACHE_HOME") == NULL ? g_build_filename(g_get_home_dir(), ".cache", NULL) : g_strdup(getenv("XDG_CACHE_HOME")));
 
-    aud_paths[BMP_PATH_USER_DIR] =
-        g_build_filename(xdg_config_home, "audacious", NULL);
-    aud_paths[BMP_PATH_USER_SKIN_DIR] =
-        g_build_filename(xdg_data_home, "audacious", "Skins", NULL);
-    aud_paths[BMP_PATH_USER_PLUGIN_DIR] =
-        g_build_filename(xdg_data_home, "audacious", "Plugins", NULL);
+    aud_paths[BMP_PATH_USER_DIR] = g_build_filename(xdg_config_home, "audacious", NULL);
+    aud_paths[BMP_PATH_USER_SKIN_DIR] = g_build_filename(xdg_data_home, "audacious", "Skins", NULL);
+    aud_paths[BMP_PATH_USER_PLUGIN_DIR] = g_build_filename(xdg_data_home, "audacious", "Plugins", NULL);
 
-    aud_paths[BMP_PATH_SKIN_THUMB_DIR] =
-        g_build_filename(xdg_cache_home, "audacious", "thumbs", NULL);
+    aud_paths[BMP_PATH_SKIN_THUMB_DIR] = g_build_filename(xdg_cache_home, "audacious", "thumbs", NULL);
 
-    aud_paths[BMP_PATH_PLAYLISTS_DIR] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR], "playlists", NULL);
+    aud_paths[BMP_PATH_PLAYLISTS_DIR] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "playlists", NULL);
 
-    aud_paths[BMP_PATH_CONFIG_FILE] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR], "config", NULL);
+    aud_paths[BMP_PATH_CONFIG_FILE] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "config", NULL);
 #ifdef HAVE_XSPF_PLAYLIST
-    aud_paths[BMP_PATH_PLAYLIST_FILE] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR],
-        "playlist.xspf", NULL);
+    aud_paths[BMP_PATH_PLAYLIST_FILE] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "playlist.xspf", NULL);
 #else
-    aud_paths[BMP_PATH_PLAYLIST_FILE] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR],
-        "playlist.m3u", NULL);
+    aud_paths[BMP_PATH_PLAYLIST_FILE] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "playlist.m3u", NULL);
 #endif
-    aud_paths[BMP_PATH_ACCEL_FILE] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR], "accels", NULL);
-    aud_paths[BMP_PATH_LOG_FILE] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR], "log", NULL);
+    aud_paths[BMP_PATH_ACCEL_FILE] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "accels", NULL);
+    aud_paths[BMP_PATH_LOG_FILE] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "log", NULL);
 
-    aud_paths[BMP_PATH_GTKRC_FILE] =
-        g_build_filename(aud_paths[BMP_PATH_USER_DIR], "gtkrc", NULL);
+    aud_paths[BMP_PATH_GTKRC_FILE] = g_build_filename(aud_paths[BMP_PATH_USER_DIR], "gtkrc", NULL);
 
     g_free(xdg_config_home);
     g_free(xdg_data_home);
@@ -185,16 +150,6 @@ aud_init_paths()
     g_atexit(aud_free_paths);
 }
 
-static void
-aud_set_default_icon(void)
-{
-    GdkPixbuf *icon;
-
-    icon = gdk_pixbuf_new_from_xpm_data((const gchar **) audacious_player_xpm);
-    gtk_window_set_default_icon(icon);
-    g_object_unref(icon);
-}
-
 static GOptionEntry cmd_entries[] = {
     {"rew", 'r', 0, G_OPTION_ARG_NONE, &options.rew, N_("Skip backwards in playlist"), NULL},
     {"play", 'p', 0, G_OPTION_ARG_NONE, &options.play, N_("Start playing current playlist"), NULL},
@@ -211,14 +166,13 @@ static GOptionEntry cmd_entries[] = {
     {"version", 'v', 0, G_OPTION_ARG_NONE, &options.version, N_("Show version"), NULL},
     {"interface", 'i', 0, G_OPTION_ARG_STRING, &options.interface, N_("Interface to use (use -i list to view available interfaces)"), NULL},
 #ifdef GDK_WINDOWING_QUARTZ
-    {"macpack", 'n', 0, G_OPTION_ARG_NONE, &options.macpack, N_("Used in macpacking"), NULL}, /* Make this hidden */
+    {"macpack", 'n', 0, G_OPTION_ARG_NONE, &options.macpack, N_("Used in macpacking"), NULL},   /* Make this hidden */
 #endif
     {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &options.filenames, N_("FILE..."), NULL},
     {NULL},
 };
 
-static void
-parse_cmd_line_options(gint *argc, gchar ***argv)
+static void parse_cmd_line_options(gint * argc, gchar *** argv)
 {
     GOptionContext *context;
     GError *error = NULL;
@@ -227,8 +181,8 @@ parse_cmd_line_options(gint *argc, gchar ***argv)
     options.session = -1;
 
     /* If audacious2 is run with no arguments, bring it to the top. This is
-     handy when the user forgets that Audacious is already running. */
-    if (* argc == 1)
+       handy when the user forgets that Audacious is already running. */
+    if (*argc == 1)
         options.mainwin = 1;
 
     context = g_option_context_new(_("- play multimedia files"));
@@ -237,21 +191,23 @@ parse_cmd_line_options(gint *argc, gchar ***argv)
 #ifdef USE_EGGSM
     g_option_context_add_group(context, egg_sm_client_get_option_group());
 #endif
+
     if (!g_option_context_parse(context, argc, argv, &error))
-        /* checking for MacOS X -psn_0_* errors*/
-        if (error->message && !g_strrstr(error->message,"-psn_0_"))
+        /* checking for MacOS X -psn_0_* errors */
+        if (error->message && !g_strrstr(error->message, "-psn_0_"))
         {
-            g_printerr(_("%s: %s\nTry `%s --help' for more information.\n"),
-                       (*argv)[0], error->message, (*argv)[0]);
+            g_printerr(_("%s: %s\nTry `%s --help' for more information.\n"), (*argv)[0], error->message, (*argv)[0]);
             exit(EXIT_FAILURE);
         }
+
+    g_free(context);
 }
 
-static void
-handle_cmd_line_filenames(gboolean is_running)
+static void handle_cmd_line_filenames(gboolean is_running)
 {
-  char * working, * absolute, * uri;
-    gchar **filenames = options.filenames;
+    gint i;
+    gchar *working, **filenames = options.filenames;
+    GList * fns = NULL;
 #ifdef USE_DBUS
     DBusGProxy *session = audacious_get_dbus_proxy();
 #endif
@@ -259,139 +215,136 @@ handle_cmd_line_filenames(gboolean is_running)
     if (filenames == NULL)
         return;
 
-    gint pos = 0;
-    gint i = 0;
-    GList *fns = NULL;
-
-    working = g_get_current_dir ();
-    for (i = 0; filenames [i]; i ++) {
-       if (strstr (filenames [i], "://"))
-          uri = g_strdup (filenames [i]);
-       else if (filenames [i] [0] == '/')
-          uri = g_filename_to_uri (filenames [i], 0, 0);
-       else {
-          absolute = g_strdup_printf ("/%s/%s", working, filenames [i]);
-          uri = g_filename_to_uri (absolute, 0, 0);
-          g_free (absolute);
-       }
-       fns = g_list_prepend (fns, uri);
+    working = g_get_current_dir();
+    for (i = 0; filenames[i] != NULL; i++)
+    {
+        gchar *uri;
+        if (strstr(filenames[i], "://"))
+            uri = g_strdup(filenames[i]);
+        else if (g_path_is_absolute(filenames[i]))
+            uri = g_filename_to_uri(filenames[i], NULL, NULL);
+        else
+        {
+            gchar *absolute = g_build_filename(working, filenames[i], NULL);
+            uri = g_filename_to_uri(absolute, NULL, NULL);
+            g_free(absolute);
+        }
+        fns = g_list_prepend(fns, uri);
     }
-    fns = g_list_reverse (fns);
-    g_free (working);
+    fns = g_list_reverse(fns);
+    g_free(working);
 
 #ifdef USE_DBUS
     if (is_running)
     {
-        GList *i;
-
         if (options.enqueue_to_temp)
-            audacious_remote_playlist_enqueue_to_temp(session, filenames[0]);
-
-        if (options.enqueue && options.play)
-            pos = audacious_remote_get_playlist_length(session);
-
-        if (!options.enqueue)
-        {
-            audacious_remote_playlist_clear(session);
-            audacious_remote_stop(session);
-        }
-
-        for (i = fns; i != NULL; i = i->next)
-            audacious_remote_playlist_add_url_string(session, i->data);
-
-        if (options.enqueue && options.play &&
-            audacious_remote_get_playlist_length(session) > pos)
-            audacious_remote_set_playlist_pos(session, pos);
-
-        if (!options.enqueue)
-            audacious_remote_play(session);
+            audacious_remote_playlist_open_list_to_temp (session, fns);
+        else if (options.enqueue)
+            audacious_remote_playlist_add (session, fns);
+        else
+            audacious_remote_playlist_open_list (session, fns);
     }
-    else /* !is_running */
+    else                        /* !is_running */
 #endif
     {
         if (options.enqueue_to_temp)
-            drct_pl_enqueue_to_temp(filenames[0]);
-
-        if (options.enqueue && options.play)
-            pos = drct_pl_get_length();
-
-        if (!options.enqueue)
         {
-            drct_pl_clear();
-            drct_stop();
-            start_playback = 1;
+            drct_pl_temp_open_list (fns);
+            cfg.resume_state = 0;
         }
-
-        drct_pl_add(fns);
-
-        if (options.enqueue && options.play &&
-            drct_pl_get_length() > pos)
-            drct_pl_set_pos(pos);
-    } /* !is_running */
+        else if (options.enqueue)
+            drct_pl_add (fns);
+        else
+        {
+            drct_pl_open_list (fns);
+            cfg.resume_state = 0;
+        }
+    }                           /* !is_running */
 
     g_list_foreach(fns, (GFunc) g_free, NULL);
     g_list_free(fns);
 }
 
-static void handle_cmd_line_options_first (void) {
+static void handle_cmd_line_options_first(void)
+{
 #ifdef USE_DBUS
-  DBusGProxy * session;
+    DBusGProxy *session;
 #endif
-   if (options.version) {
-      print_version ();
-      exit (EXIT_SUCCESS);
-   }
+
+    if (options.version)
+    {
+        print_version();
+        exit(EXIT_SUCCESS);
+    }
+
 #ifdef USE_DBUS
-   session = audacious_get_dbus_proxy ();
-   if (audacious_remote_is_running (session)) {
-      handle_cmd_line_filenames (TRUE);
-      if (options.rew)
-         audacious_remote_playlist_prev (session);
-      if (options.play)
-         audacious_remote_play (session);
-      if (options.pause)
-         audacious_remote_pause (session);
-      if (options.stop)
-         audacious_remote_stop (session);
-      if (options.fwd)
-         audacious_remote_playlist_next (session);
-      if (options.play_pause)
-         audacious_remote_play_pause (session);
-      if (options.show_jump_box)
-         audacious_remote_show_jtf_box (session);
-      if (options.mainwin)
-         audacious_remote_main_win_toggle (session, 1);
-      if (options.activate)
-         audacious_remote_activate (session);
-      exit (EXIT_SUCCESS);
-   }
+    session = audacious_get_dbus_proxy();
+    if (audacious_remote_is_running(session))
+    {
+        handle_cmd_line_filenames(TRUE);
+        if (options.rew)
+            audacious_remote_playlist_prev(session);
+        if (options.play)
+            audacious_remote_play(session);
+        if (options.pause)
+            audacious_remote_pause(session);
+        if (options.stop)
+            audacious_remote_stop(session);
+        if (options.fwd)
+            audacious_remote_playlist_next(session);
+        if (options.play_pause)
+            audacious_remote_play_pause(session);
+        if (options.show_jump_box)
+            audacious_remote_show_jtf_box(session);
+        if (options.mainwin)
+            audacious_remote_main_win_toggle(session, 1);
+        if (options.activate)
+            audacious_remote_activate(session);
+        exit(EXIT_SUCCESS);
+    }
 #endif
-   if (options.interface == NULL) {
-     mcs_handle_t * db = cfg_db_open ();
-      cfg_db_get_string (db, NULL, "interface", & options.interface);
-      if (options.interface == NULL)
-         options.interface = g_strdup ("skinned");
-   }
+
+    if (options.interface == NULL)
+    {
+        mcs_handle_t *db = cfg_db_open();
+        cfg_db_get_string(db, NULL, "interface", &options.interface);
+        if (options.interface == NULL)
+            options.interface = g_strdup("skinned");
+        cfg_db_close(db);
+    }
 }
 
-static void handle_cmd_line_options (void) {
-   handle_cmd_line_filenames (FALSE);
-   if (options.rew)
-      drct_pl_prev ();
-   if (options.play || options.play_pause)
-       start_playback = 1;
-   if (options.fwd)
-      drct_pl_next ();
-   if (options.show_jump_box)
-      drct_jtf_show ();
-   if (options.mainwin)
-      drct_main_win_toggle (TRUE);
-   if (options.activate)
-      drct_activate ();
+static void handle_cmd_line_options(void)
+{
+    handle_cmd_line_filenames(FALSE);
+
+    if (cfg.resume_playback_on_startup && cfg.resume_state > 0)
+    {
+        playback_initiate ();
+
+        if (cfg.resume_state == 2)
+            playback_pause ();
+
+        playback_seek (cfg.resume_playback_on_startup_time);
+    }
+
+    if (options.play || options.play_pause)
+    {
+        if (! playback_get_playing ())
+            playback_initiate ();
+        else if (playback_get_paused ())
+            playback_pause ();
+    }
+
+    if (options.show_jump_box)
+        drct_jtf_show();
+    if (options.mainwin)
+        drct_main_win_toggle(TRUE);
+    if (options.activate)
+        drct_activate();
 }
 
-static void
-aud_setup_logger(void)
+static void aud_setup_logger(void)
 {
     if (!aud_logger_start(aud_paths[BMP_PATH_LOG_FILE]))
         return;
@@ -399,85 +352,110 @@ aud_setup_logger(void)
     g_atexit(aud_logger_stop);
 }
 
-static gboolean
-load_extra_playlist(const gchar * path, const gchar * basename,
-        gpointer def)
+static gboolean load_extra_playlist(const gchar * path, const gchar * basename, gpointer def)
 {
-    Playlist *playlist = playlist_new();
+    gchar * filename = g_filename_to_uri (path, NULL, NULL);
+    gint playlist = playlist_count();
 
-    if (!playlist) {
-        g_warning("Couldn't create new playlist for %s / %s\n", path, basename);
-        return FALSE;
-    }
+    playlist_insert(playlist);
+    playlist_insert_playlist(playlist, 0, filename);
 
-    playlist_add_playlist(playlist);
-    playlist_load(playlist, path);
+    g_free (filename);
 
-    return FALSE; /* keep loading other playlists */
+    return FALSE;               /* keep loading other playlists */
 }
 
-static void
-playlist_system_init()
+static void playlist_system_init()
 {
-    Playlist *playlist;
+    gchar * filename = g_filename_to_uri (aud_paths[BMP_PATH_PLAYLIST_FILE],
+     NULL, NULL);
 
     playlist_init();
-    playlist = playlist_get_active();
-    playlist_load(playlist, aud_paths[BMP_PATH_PLAYLIST_FILE]);
-    playlist_set_position(playlist, cfg.playlist_position);
+
+    if (vfs_file_test (filename, G_FILE_TEST_EXISTS))
+        playlist_insert_playlist (0, 0, filename);
+
+    g_free (filename);
 
     /* Load extra playlists */
-    if (!dir_foreach(aud_paths[BMP_PATH_PLAYLISTS_DIR], load_extra_playlist,
-                     playlist, NULL))
+    if (!dir_foreach(aud_paths[BMP_PATH_PLAYLISTS_DIR], load_extra_playlist, NULL, NULL))
         g_warning("Could not load extra playlists\n");
+
+    playlist_load_state ();
+    playlist_set_shuffle(cfg.shuffle);
 }
 
-void
-aud_quit(void)
+void aud_quit(void)
 {
     Interface *i = interface_get(options.interface);
 
     g_message("Saving configuration");
     aud_config_save();
+    playlist_save_state ();
+
+    if (playback_get_playing ())
+        playback_stop ();
 
     g_message("Shutting down user interface subsystem");
     interface_destroy(i);
 
+    output_cleanup ();
+
     g_message("Plugin subsystem shutdown");
     plugin_system_cleanup();
 
     g_message("Playlist cleanup");
-    playlist_end ();
-    
-    g_message("Playlist scanner shutdown");
-    scanner_end ();
+    playlist_end();
 
     g_message("Shutdown finished, bye.");
     exit(EXIT_SUCCESS);
 }
 
-static
-int print_interface_info(mowgli_dictionary_elem_t *delem, void *privdata)
+static int print_interface_info(mowgli_dictionary_elem_t * delem, void *privdata)
 {
     Interface *i = (Interface *) delem->data;
     g_print("  %-15s - %s\n", i->id, i->desc);
     return 0;
 }
 
-static void quit_cb (void * hook_data, void * user_data)
+static void quit_cb(void *hook_data, void *user_data)
 {
-    aud_quit ();
+    aud_quit();
 }
 
-gint
-main(gint argc, gchar ** argv)
+#ifdef USE_DBUS
+static void mpris_status_cb1(gpointer hook_data, gpointer user_data)
+{
+    mpris_emit_status_change(mpris, GPOINTER_TO_INT(user_data));
+}
+
+static void mpris_status_cb2(gpointer hook_data, gpointer user_data)
+{
+    mpris_emit_status_change(mpris, -1);
+}
+
+void init_playback_hooks(void)
+{
+    hook_associate("playback begin", mpris_status_cb1, GINT_TO_POINTER(MPRIS_STATUS_PLAY));
+    hook_associate("playback pause", mpris_status_cb1, GINT_TO_POINTER(MPRIS_STATUS_PAUSE));
+    hook_associate("playback unpause", mpris_status_cb1, GINT_TO_POINTER(MPRIS_STATUS_PLAY));
+    hook_associate("playback stop", mpris_status_cb1, GINT_TO_POINTER(MPRIS_STATUS_STOP));
+
+    hook_associate("playback shuffle", mpris_status_cb2, NULL);
+    hook_associate("playback repeat", mpris_status_cb2, NULL);
+    hook_associate("playback no playlist advance", mpris_status_cb2, NULL);
+}
+#endif
+
+gint main(gint argc, gchar ** argv)
 {
     Interface *i;
 
     /* glib-2.13.0 requires g_thread_init() to be called before all
        other GLib functions */
     g_thread_init(NULL);
-    if (!g_thread_supported()) {
+    if (!g_thread_supported())
+    {
         g_printerr(_("Sorry, threads aren't supported on your platform.\n"));
         exit(EXIT_FAILURE);
     }
@@ -486,8 +464,8 @@ main(gint argc, gchar ** argv)
     mowgli_init();
     chardet_init();
 
-    hook_init ();
-    hook_associate ("quit", quit_cb, 0);
+    hook_init();
+    hook_associate("quit", quit_cb, 0);
 
     /* Setup l10n early so we can print localized error messages */
     gtk_set_locale();
@@ -511,7 +489,8 @@ main(gint argc, gchar ** argv)
         aud_setup_logger();
 
     g_message("Initializing Gtk+");
-    if (!gtk_init_check(&argc, &argv)) { /* XXX */
+    if (!gtk_init_check(&argc, &argv))
+    {                           /* XXX */
         /* GTK check failed, and no arguments passed to indicate
            that user is intending to only remote control a running
            session */
@@ -519,10 +498,6 @@ main(gint argc, gchar ** argv)
         exit(EXIT_FAILURE);
     }
 
-    g_message("Setting up libSAD");
-    g_random_set_seed(time(NULL));
-    SAD_dither_init_rand((gint32)time(NULL));
-
     g_message("Loading configuration");
     aud_config_load();
 
@@ -530,23 +505,14 @@ main(gint argc, gchar ** argv)
     signal_handlers_init();
 
     g_message("Handling commandline options, part #1");
-    handle_cmd_line_options_first ();
+    handle_cmd_line_options_first();
 
-    if (g_ascii_strcasecmp(options.interface, "headless")) /* XXX */
-    {
-        g_message("Non-headless operation setup");
-
-        // register icons in stock
-        register_aud_stock_icons();
-
-        create_prefs_window();
-    }
-
-    vis_runner_init ();
+    output_init ();
 
 #ifdef USE_DBUS
     g_message("Initializing D-Bus");
     init_dbus();
+    init_playback_hooks();
 #endif
 
     g_message("Initializing plugin subsystems...");
@@ -556,7 +522,8 @@ main(gint argc, gchar ** argv)
     ui_populate_headless_interface();
 
     /* Check if user wants to list available interfaces */
-    if (!g_ascii_strcasecmp(options.interface, "list")) {
+    if (!g_ascii_strcasecmp(options.interface, "list"))
+    {
         g_print(_("Available interfaces:\n\n"));
         interface_foreach(print_interface_info, NULL);
         g_print("\n");
@@ -564,30 +531,14 @@ main(gint argc, gchar ** argv)
         exit(EXIT_SUCCESS);
     }
 
-    scanner_init ();
-    scanner_enable (cfg.use_pl_metadata);
     playlist_system_init();
-
-    g_message("Handling commandline options, part #2");
-    handle_cmd_line_options ();
-
     init_equalizer ();
 
-    if (start_playback)
-        playback_initiate ();
-    else if (cfg.resume_playback_on_startup && cfg.resume_state)
-    {
-        playback_initiate ();
-
-        if (cfg.resume_state == 2)
-            playback_pause ();
-
-        playback_seek (cfg.resume_playback_on_startup_time);
-    }
-
-    g_message("Setting default icon");
-    aud_set_default_icon();
+    g_message("Handling commandline options, part #2");
+    handle_cmd_line_options();
 
+    g_message("Registering interface hooks");
+    register_interface_hooks();
 
 #ifndef NOT_ALPHA_RELEASE
     g_message("Displaying unsupported version warning.");
@@ -599,7 +550,7 @@ main(gint argc, gchar ** argv)
 
     if (i != NULL)
     {
-        g_message("Running interface %s@%p", options.interface, i);
+        g_message("Running interface %s@%p", options.interface, (void *) i);
         interface_run(i);
     }
     else
