@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2007  Audacious development team
+ *  Copyright (C) 2005-2009  Audacious development team
  *
  *  Based on BMP:
  *  Copyright (C) 2003-2004  BMP development team
@@ -30,45 +30,17 @@
 
 #include "plugin.h"
 
-typedef struct _OutputPluginData OutputPluginData;
-
-struct _OutputPluginData {
-    GList *output_list;
-    OutputPlugin *current_output_plugin;
-};
-
-typedef struct _OutputPluginState OutputPluginState;
-
-struct _OutputPluginState {
-    AFormat fmt;
-    gint rate;
-    gint nch;
-};
+extern struct OutputAPI output_api;
+extern OutputPlugin * current_output_plugin;
 
 GList *get_output_list(void);
-OutputPlugin *get_current_output_plugin(void);
-void set_current_output_plugin(gint i);
-void output_about(gint i);
-void output_configure(gint i);
 void output_get_volume(gint * l, gint * r);
 void output_set_volume(gint l, gint r);
-void output_set_eq(gboolean, gfloat, gfloat *);
-gint output_open_audio(AFormat, gint, gint);
-void output_write_audio(gpointer ptr, gint length);
-void output_close_audio(void);
 
-void output_flush(gint);
-void output_pause(gshort);
-gint output_buffer_free(void);
-gint output_buffer_playing(void);
-
-void output_pass_audio(InputPlayback *, AFormat, gint, gint, gpointer, int *);
-void output_set_replaygain_info(InputPlayback *, ReplayGainInfo *);
-
-gint get_written_time(void);
-gint get_output_time(void);
-
-extern OutputPlugin psuedo_output_plugin;
-extern OutputPluginData op_data;
+void output_init (void);
+void output_cleanup (void);
+void set_current_output_plugin (OutputPlugin * plugin);
+gint get_output_time (void);
+void output_drain (void);
 
 #endif /* AUDACIOUS_OUTPUT_H */
