@@ -25,21 +25,22 @@
 #ifndef AUDACIOUS_PLUGINENUM_H
 #define AUDACIOUS_PLUGINENUM_H
 
-#include <glib.h>
 #include "config.h"
 
+#include <glib.h>
+
+#include "plugin.h"
+
 #define PLUGIN_FILENAME(name) (name SHARED_SUFFIX)
 
 void plugin_system_init(void);
 void plugin_system_cleanup(void);
 
+void module_load (const gchar * path);
+
 extern const gchar *plugin_dir_list[];
 extern GHashTable *plugin_matrix;
 
-gboolean plugin_is_enabled(const gchar *filename);
-void plugin_set_enabled(const gchar *filename, gboolean enabled);
-Plugin *plugin_get_plugin(const gchar *filename);
-
 void plugin_set_current(Plugin *plugin);
 
 #endif /* AUDACIOUS_PLUGINENUM_H */
