@@ -24,6 +24,7 @@
 #  include "config.h"
 #endif
 
+#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -40,31 +41,26 @@
 
 #include "plugin.h"
 #include "pluginenum.h"
-#include "input.h"
 #include "effect.h"
-#include "strings.h"
+#include "audstrings.h"
 #include "general.h"
 #include "output.h"
+#include "playlist-new.h"
+#include "probe.h"
 #include "visualization.h"
 
 #include "main.h"
 #include "util.h"
-#include "dnd.h"
 #include "tuple.h"
 #include "vfs.h"
 
-#include "playlist.h"
-
-#include "ui_main.h"
-#include "ui_playlist.h"
 #include "build_stamp.h"
 #include "ui_fileinfo.h"
-#include "ui_playlist.h"
 
 #define G_FREE_CLEAR(a) if(a != NULL) { g_free(a); a = NULL; }
 #define STATUS_TIMEOUT 3*1000
 
-GtkWidget *fileinfo_win;
+GtkWidget *fileinfo_win = NULL;
 
 GtkWidget *entry_location;
 GtkWidget *entry_title;
@@ -264,7 +260,8 @@ fileinfo_entry_set_image(GtkWidget *widget, const char *text)
     g_object_unref(G_OBJECT(pixbuf));
 }
 
-void fileinfo_hide(gpointer unused)
+static int
+fileinfo_hide(gpointer unused)
 {
     if(GTK_WIDGET_VISIBLE(fileinfo_win)) gtk_widget_hide(fileinfo_win);
 
@@ -281,12 +278,12 @@ void fileinfo_hide(gpointer unused)
     fileinfo_label_set_text(label_format_name, NULL);
     fileinfo_label_set_text(label_quality, NULL);
     fileinfo_label_set_text(label_bitrate, NULL);
-    
+
     if (label_mini_status != NULL) {
         gtk_label_set_text(GTK_LABEL(label_mini_status), "<span size=\"small\"></span>");
         gtk_label_set_use_markup(GTK_LABEL(label_mini_status), TRUE);
     }
-    
+
     something_changed = FALSE;
     gtk_widget_set_sensitive(btn_apply, FALSE);
 
@@ -294,9 +291,11 @@ void fileinfo_hide(gpointer unused)
     G_FREE_CLEAR(current_file);
 
     fileinfo_entry_set_image(image_artwork, DATA_DIR "/images/audio.png");
+    return 1;
 }
 
-static void entry_changed (GtkEditable *editable, gpointer user_data)
+static void
+entry_changed (GtkEditable *editable, gpointer user_data)
 {
     if(current_file != NULL && current_ip != NULL && current_ip->update_song_tuple != NULL) {
         something_changed = TRUE;
@@ -396,7 +395,7 @@ themed_icon_lookup(gint size, const gchar *name, ...)
 
     if (pixbuf != NULL)
         return pixbuf;
-    
+
     if (error != NULL)
         g_error_free(error);
 
@@ -414,7 +413,7 @@ themed_icon_lookup(gint size, const gchar *name, ...)
         if (error != NULL)
             g_error_free(error);
     }
-    
+
     return NULL;
 }
 
@@ -452,6 +451,18 @@ mime_icon_lookup(gint size, const gchar *mime_type) /* smart icon resolving rout
     return icon;
 }
 
+static gboolean fileinfo_keypress (GtkWidget * widget, GdkEventKey * event,
+ void * unused)
+{
+    if (event->keyval == GDK_Escape)
+    {
+        fileinfo_hide (NULL);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
 void
 create_fileinfo_window(void)
 {
@@ -489,9 +500,7 @@ create_fileinfo_window(void)
     gtk_container_set_border_width(GTK_CONTAINER(fileinfo_win), 6);
     gtk_window_set_title(GTK_WINDOW(fileinfo_win), _("Track Information"));
     gtk_window_set_position(GTK_WINDOW(fileinfo_win), GTK_WIN_POS_CENTER);
-    gtk_window_set_resizable(GTK_WINDOW(fileinfo_win), FALSE);
     gtk_window_set_type_hint(GTK_WINDOW(fileinfo_win), GDK_WINDOW_TYPE_HINT_DIALOG);
-    gtk_window_set_transient_for(GTK_WINDOW(fileinfo_win), GTK_WINDOW(mainwin));
 
     vbox0 = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(fileinfo_win), vbox0);
@@ -520,12 +529,12 @@ create_fileinfo_window(void)
 
     vbox3 = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(alignment), vbox3);
-    
+
     label_general = gtk_label_new(_("<span size=\"small\">General</span>"));
     gtk_box_pack_start (GTK_BOX (vbox2), label_general, FALSE, FALSE, 0);
     gtk_label_set_use_markup(GTK_LABEL(label_general), TRUE);
     gtk_misc_set_alignment(GTK_MISC(label_general), 0, 0.5);
-    
+
     alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
     gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 6, 0, 0);
     gtk_box_pack_start (GTK_BOX (vbox2), alignment, FALSE, FALSE, 0);
@@ -535,7 +544,7 @@ create_fileinfo_window(void)
 
     image_fileicon = gtk_image_new_from_stock (GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG);
     gtk_box_pack_start (GTK_BOX (codec_hbox), image_fileicon, FALSE, FALSE, 0);
-    
+
     codec_table = gtk_table_new(3, 2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE(codec_table), 6);
     gtk_table_set_col_spacings (GTK_TABLE(codec_table), 12);
@@ -560,7 +569,7 @@ create_fileinfo_window(void)
     label_bitrate = gtk_label_new(_("<span size=\"small\">n/a</span>"));
     gtk_label_set_use_markup(GTK_LABEL(label_bitrate), TRUE);
     gtk_misc_set_alignment(GTK_MISC(label_bitrate), 0, 0.5);
-    
+
     gtk_table_attach(GTK_TABLE(codec_table), label_format, 0, 1, 0, 1,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);
@@ -643,7 +652,7 @@ create_fileinfo_window(void)
 
         for (i = 0; i < G_N_ELEMENTS(genre_table); i++)
             genre_list = g_list_prepend(genre_list, _(genre_table[i]));
-        genre_list = g_list_sort(genre_list, (GCompareFunc) g_ascii_strcasecmp);
+        genre_list = g_list_sort(genre_list, (GCompareFunc) g_utf8_collate);
 
         MOWGLI_ITER_FOREACH(iter, genre_list)
             gtk_combo_box_append_text(GTK_COMBO_BOX(entry_genre), iter->data);
@@ -746,7 +755,7 @@ create_fileinfo_window(void)
     gtk_tree_view_column_set_attributes(column, renderer,
                                         "text", RAWDATA_VALUE, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(treeview_rawdata), column);
-    
+
     hbox_status_and_bbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox0), hbox_status_and_bbox, FALSE, FALSE, 0);
 
@@ -754,7 +763,7 @@ create_fileinfo_window(void)
     gtk_label_set_use_markup(GTK_LABEL(label_mini_status), TRUE);
     gtk_misc_set_alignment(GTK_MISC(label_mini_status), 0, 0.5);
     gtk_box_pack_start (GTK_BOX (hbox_status_and_bbox), label_mini_status, TRUE, TRUE, 0);
-    
+
     bbox_close = gtk_hbutton_box_new();
     gtk_box_set_spacing(GTK_BOX(bbox_close), 6);
     gtk_box_pack_start(GTK_BOX(hbox_status_and_bbox), bbox_close, FALSE, FALSE, 0);
@@ -769,14 +778,18 @@ create_fileinfo_window(void)
     gtk_container_add(GTK_CONTAINER(bbox_close), btn_close);
     GTK_WIDGET_SET_FLAGS(btn_close, GTK_CAN_DEFAULT);
     g_signal_connect(G_OBJECT(btn_close), "clicked", (GCallback) fileinfo_hide, NULL);
+    g_signal_connect ((GObject *) fileinfo_win, "delete-event", (GCallback)
+     fileinfo_hide, NULL);
+    g_signal_connect ((GObject *) fileinfo_win, "key-press-event", (GCallback)
+     fileinfo_keypress, NULL);
 
     gtk_widget_show_all (vbox0);
 }
 
-void 
+static void
 fileinfo_show_for_tuple(Tuple *tuple, gboolean updating_enabled)
 {
-    gchar *tmp, *tmp_utf = NULL;
+    gchar *tmp = NULL;
     GdkPixbuf *icon = NULL;
     GtkTreeIter iter;
     GtkListStore *store;
@@ -786,7 +799,7 @@ fileinfo_show_for_tuple(Tuple *tuple, gboolean updating_enabled)
 
     if (tuple == NULL)
         return;
-        
+
     if(!updating_enabled) {
         current_ip = NULL;
         G_FREE_CLEAR(current_file);
@@ -794,6 +807,9 @@ fileinfo_show_for_tuple(Tuple *tuple, gboolean updating_enabled)
 
     something_changed = FALSE;
 
+    if (fileinfo_win == NULL)
+        create_fileinfo_window();
+
     if (!GTK_WIDGET_REALIZED(fileinfo_win))
         gtk_widget_realize(fileinfo_win);
 
@@ -803,18 +819,15 @@ fileinfo_show_for_tuple(Tuple *tuple, gboolean updating_enabled)
     set_entry_str_from_field(entry_comment, tuple, FIELD_COMMENT, updating_enabled);
     set_entry_str_from_field(gtk_bin_get_child(GTK_BIN(entry_genre)), tuple, FIELD_GENRE, updating_enabled);
 
-    tmp = g_strdup_printf("%s/%s",
+    tmp = g_strdup_printf ("%s%s",
             tuple_get_string(tuple, FIELD_FILE_PATH, NULL),
             tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
 
     if (tmp) {
-        tmp_utf = str_to_utf8(tmp);
-        fileinfo_entry_set_text(entry_location, tmp_utf);
-        g_free(tmp_utf);
+        fileinfo_entry_set_text(entry_location, tmp);
         g_free(tmp);
-        tmp = NULL;
     }
-        
+
     /* set empty string if field not availaible. --eugene */
     set_entry_int_from_field(entry_year, tuple, FIELD_YEAR, updating_enabled);
     set_entry_int_from_field(entry_track, tuple, FIELD_TRACK_NUMBER, updating_enabled);
@@ -839,14 +852,14 @@ fileinfo_show_for_tuple(Tuple *tuple, gboolean updating_enabled)
     tmp = fileinfo_recursive_get_image(
             tuple_get_string(tuple, FIELD_FILE_PATH, NULL),
             tuple_get_string(tuple, FIELD_FILE_NAME, NULL), 0);
-        
+
     if (tmp) {
         fileinfo_entry_set_image(image_artwork, tmp);
         g_free(tmp);
     }
 
     gtk_widget_set_sensitive(btn_apply, FALSE);
-    
+
     if (label_mini_status != NULL) {
         gtk_label_set_text(GTK_LABEL(label_mini_status), "<span size=\"small\"></span>");
         gtk_label_set_use_markup(GTK_LABEL(label_mini_status), TRUE);
@@ -907,23 +920,7 @@ fileinfo_show_for_tuple(Tuple *tuple, gboolean updating_enabled)
         gtk_widget_show(fileinfo_win);
 }
 
-void
-fileinfo_show_for_path(gchar *path)
-{
-    Tuple *tuple = input_get_song_tuple(path);
-
-    if (tuple == NULL) {
-         input_file_info_box(path);
-         return;
-    }
-
-    fileinfo_show_for_tuple(tuple, FALSE);
-
-    mowgli_object_unref(tuple);
-}
-
-void
-fileinfo_show_editor_for_path(gchar *path, InputPlugin *ip)
+static void fileinfo_show_editor_for_path (const gchar * path, InputPlugin * ip)
 {
     G_FREE_CLEAR(current_file);
     current_file = g_strdup(path);
@@ -940,3 +937,57 @@ fileinfo_show_editor_for_path(gchar *path, InputPlugin *ip)
 
     mowgli_object_unref(tuple);
 }
+
+void ui_fileinfo_show (gint playlist, gint entry)
+{
+    const gchar * filename = playlist_entry_get_filename (playlist, entry);
+    InputPlugin * decoder = playlist_entry_get_decoder (playlist, entry);
+    Tuple * tuple;
+
+    g_return_if_fail (filename != NULL);
+
+    if (decoder == NULL)
+        decoder = file_probe (filename, FALSE);
+
+    if (decoder == NULL)
+    {
+        gchar * message = g_strdup_printf ("No decoder found for %s.\n",
+         filename);
+
+        hook_call ("interface show error", message);
+        g_free (message);
+        return;
+    }
+
+    if (decoder->file_info_box != NULL)
+        decoder->file_info_box (filename);
+    else if (decoder->update_song_tuple != NULL && ! vfs_is_remote (filename))
+        fileinfo_show_editor_for_path (filename, decoder);
+    else if ((tuple = (Tuple *) playlist_entry_get_tuple (playlist, entry)) !=
+     NULL)
+        fileinfo_show_for_tuple (tuple, FALSE);
+    else if (decoder->get_song_tuple != NULL && (tuple = decoder->get_song_tuple
+     (filename)) != NULL)
+    {
+        fileinfo_show_for_tuple (tuple, FALSE);
+        playlist_entry_set_tuple (playlist, entry, tuple);
+    }
+    else
+    {
+        gchar * message = g_strdup_printf ("No info available for "
+         "%s.\n", filename);
+
+        hook_call ("interface show error", message);
+        g_free (message);
+    }
+}
+
+void ui_fileinfo_show_current (void)
+{
+    gint playlist = playlist_get_playing ();
+
+    if (playlist == -1)
+        playlist = playlist_get_active ();
+
+    ui_fileinfo_show (playlist, playlist_get_position (playlist));
+}
