@@ -38,7 +38,6 @@
 G_BEGIN_DECLS
 
 #include "audacious/plugin.h"
-#include "libSAD/libSAD.h"
 
 #define SWAP(a, b)      { a^=b; b^=a; a^=b; }
 
@@ -49,7 +48,6 @@ typedef gboolean(*DirForeachFunc) (const gchar * path,
 
 gchar *find_file_recursively(const gchar * dirname, const gchar * file);
 gchar *find_path_recursively(const gchar * dirname, const gchar * file);
-void del_directory(const gchar * dirname);
 gboolean dir_foreach(const gchar * path, DirForeachFunc function,
                      gpointer user_data, GError ** error);
 
@@ -66,38 +64,30 @@ GArray *string_to_garray(const gchar * str);
 void glist_movedown(GList * list);
 void glist_moveup(GList * list);
 
-GdkFont *util_font_load(const gchar * name);
 void util_set_cursor(GtkWidget * window);
 gboolean text_get_extents(const gchar * fontname, const gchar * text,
                           gint * width, gint * height, gint * ascent,
                           gint * descent);
 
-gboolean file_is_archive(const gchar * filename);
-gchar *archive_decompress(const gchar * path);
-gchar *archive_basename(const gchar * path);
-
 guint gint_count_digits(gint n);
 
 
-GtkWidget *make_filebrowser(const gchar *title, gboolean save);
-
 GtkWidget *util_info_dialog(const gchar * title, const gchar * text,
     const gchar * button_text, gboolean modal, GCallback button_action,
-    gpointer action_data); 
-
-GdkPixbuf *audacious_create_colorized_pixbuf(GdkPixbuf *src, gint red, gint green, gint blue);
+    gpointer action_data);
 
 gchar *util_get_localdir(void);
 
 gchar *construct_uri(gchar *string, const gchar *playlist_name);
 
-SAD_sample_format sadfmt_from_afmt(AFormat fmt);
-
 /* minimizes number of realloc's */
 gpointer smart_realloc(gpointer ptr, gsize *size);
 
+gint file_get_mtime (const gchar * filename);
 void make_directory(const gchar * path, mode_t mode);
 
+void util_add_url_history_entry(const gchar * url);
+
 G_END_DECLS
 
 #endif /* AUDACIOUS_UTIL_H */
