@@ -26,8 +26,6 @@
 #include "visualization.h"
 
 #include <glib.h>
-#include <stdint.h>
-#include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
@@ -62,8 +60,7 @@ get_vis_enabled_list(void)
 void
 vis_disable_plugin(VisPlugin * vp)
 {
-    gint i = g_list_index(vp_data.vis_list, vp);
-    enable_vis_plugin(i, FALSE);
+    vis_enable_plugin(vp, FALSE);
 }
 
 void
@@ -107,25 +104,28 @@ vis_playback_stop(void)
 }
 
 void
-enable_vis_plugin(gint i, gboolean enable)
+vis_enable_plugin(VisPlugin *vp, gboolean enable)
 {
-    GList *node = g_list_nth(vp_data.vis_list, i);
-    VisPlugin *vp;
-
-    if (!node || !(node->data))
+    if (!vp)
         return;
-    vp = node->data;
 
-    if (enable && ! vp->enabled)
+    if (enable && !vp->enabled)
     {
-        if (! vp_data.enabled_list)
-            vis_runner_add_hook (send_audio, 0);
+        if (vp_data.enabled_list == NULL)
+            vis_runner_add_hook(send_audio, 0);
 
         vp_data.enabled_list = g_list_append(vp_data.enabled_list, vp);
         if (vp->init)
         {
             plugin_set_current((Plugin *)vp);
             vp->init();
+
+            if (vp->get_widget != NULL)
+            {
+                GtkWidget *w = vp->get_widget();
+
+                interface_run_gtk_plugin(w, vp->description);
+            }
         }
         if (playback_get_playing() && vp->playback_start)
         {
@@ -140,14 +140,20 @@ enable_vis_plugin(gint i, gboolean enable)
             plugin_set_current((Plugin *)vp);
             vp->playback_stop();
         }
+        if (vp->get_widget != NULL)
+        {
+            GtkWidget *w = vp->get_widget();
+
+            interface_stop_gtk_plugin(w);
+        }
         if (vp->cleanup)
         {
             plugin_set_current((Plugin *)vp);
             vp->cleanup();
         }
 
-        if (! vp_data.enabled_list)
-            vis_runner_remove_hook (send_audio);
+        if (vp_data.enabled_list == NULL)
+            vis_runner_remove_hook(send_audio);
     }
 
     vp->enabled = enable;
@@ -172,6 +178,16 @@ vis_stringify_enabled_list(void)
     return enalist;
 }
 
+static gboolean
+vis_queued_enable_cb(gpointer vp)
+{
+    VisPlugin *p = vp;
+
+    vis_enable_plugin(p, TRUE);
+
+    return FALSE;
+}
+
 void
 vis_enable_from_stringified_list(gchar * list)
 {
@@ -186,8 +202,9 @@ vis_enable_from_stringified_list(gchar * list)
         for (node = vp_data.vis_list; node != NULL; node = g_list_next(node)) {
             base = g_path_get_basename(VIS_PLUGIN(node->data)->filename);
 
-            if (! strcmp (plugins[i], base))
-                enable_vis_plugin (i, 1);
+            /* don't start the vis plugin until our eventloop and interface is up. --nenolod */
+            if (!strcmp(plugins[i], base))
+                g_idle_add(vis_queued_enable_cb, node->data);
 
             g_free(base);
         }
@@ -274,9 +291,6 @@ static void send_audio (void * hook_data, void * user_data)
     VisNode * vis_node = hook_data;
     int16_t (* pcm_data) [512] = vis_node->data;
     int nch = vis_node->nch;
-
-    // We ignore vis_node->length?! -jlindgren
-
     GList *node = vp_data.enabled_list;
     VisPlugin *vp;
     gint16 mono_freq[2][256], stereo_freq[2][256];
